<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class extends the core Kohana class by adding some core application
 * specific functions, and configuration.
 *
 * @package    OSB
 * @category   Modifications
 * @author     Deon George
 * @copyright  (c) 2009-2013 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 */
class Config extends Kohana_Config {
	// Our default logo, if there is no site logo
	public static $logo = 'img/logo-small.png';

	/**
	 * @compat Restore KH 3.1 functionality
	 * @var  Kohana_Config  Singleton static instance
	 */
	protected static $_instance;

	/**
	 * Some early initialisation
	 *
	 * At this point, KH hasnt been fully initialised either, so we cant rely on
	 * too many KH functions yet.
	 *
	 * NOTE: Kohana doesnt provide a parent construct for the Kohana_Config class.
	 */
	public function __construct() {
		if (defined('PHPUNITTEST'))
			$_SERVER['SERVER_NAME'] = PHPUNITTEST;
	}

	/**
	 * Get the singleton instance of Config.
	 *
	 *     $config = Config::instance();
	 *
	 * @return  Config
	 * @compat Restore KH 3.1 functionality
	 */
	public static function instance() {
		if (Config::$_instance === NULL)
			// Create a new instance
			Config::$_instance = new Config;

		return Config::$_instance;
	}

	/**
	 * Return our caching mechanism
	 */
	public static function cachetype() {
		return is_null(Kohana::$config->load('config')->cache_type) ? 'file' : Kohana::$config->load('config')->cache_type;
	}

	public static function copywrite() {
		return '(c) Open Source Billing Development Team';
	}

	public static function country() {
		return Company::instance()->country();
	}

	/**
	 * Show a date using a site configured format
	 */
	public static function date($date) {
		return (is_null($date) OR ! $date) ? '' : date(Company::instance()->date_format(),$date);
	}

	/**
	 * Show a date using a site configured format
	 * @note We need this function here, since we call self:: methods, which need to resolve to the child class.
	 */
	public static function datetime($date) {
		return sprintf('%s %s',self::date($date),self::time($date));
	}

	public static function language() {
		return Company::instance()->language();
	}

	/**
	 * The URI to show for the login prompt.
	 * Normally if the user is logged in, we can replace it with something else
	 */
	public static function login_uri() {
		return ($ao = Auth::instance()->get_user() AND is_object($ao)) ? HTML::anchor(URL::link('user','account/edit'),$ao->name()) : HTML::anchor('login',_('Login'));
	}

	public static function logout_uri() {
		return ($ao = Auth::instance()->get_user() AND is_object($ao)) ? HTML::anchor('logout','Logout',array('class'=>'lnk_logout')) : '';
	}

	public static function logo() {
		return HTML::image(self::logo_uri(),array('class'=>'headlogo','alt'=>_('Logo')));
	}

	public static function logo_uri($protocol=NULL) {
		list ($path,$suffix) = explode('.',self::$logo);

		return URL::site(Route::get('default/media')->uri(array('file'=>$path.'.'.$suffix),array('alt'=>self::sitename())),$protocol);
	}

	/**
	 * Find a list of all database enabled modules
	 *
	 * Our available modules are defined in the DB (along with method
	 * security).
	 */
	public static function modules() {
		static $result = array();

		if (! count($result)) {
			// We need to know our site here, so that we can subsequently load our enabled modules.
			if (PHP_SAPI === 'cli') {
				if (! ($site = Minion_CLI::options('site'))) {
					echo _('Cant figure out the site, use --site= for CLI')."\n";
					die();

				} else
					$_SERVER['SERVER_NAME'] = $site;

			}

			foreach (ORM::factory('Module')->list_external() as $mo)
				$result[$mo->name] = MODPATH.$mo->name;
		}

		return $result;
	}

	public static function module_config($item,array $value=NULL) {
		return Company::instance()->module_config($item,$value);
	}

	public static function module_exist($module) {
		return array_key_exists(strtolower($module),self::modules()) ? TRUE : FALSE;
	}

	/**
	 * Work out our site ID for multiehosting
	 */
	public static function siteid($format=FALSE) {
		return Company::instance()->site($format);
	}

	/**
	 * Work out our site mode (dev,test,prod)
	 */
	public static function sitemode() {
		return Company::instance()->sitemode();
	}

	public static function sitename() {
		return Company::instance()->name();
	}

	/**
	 * See if our emails for the template should be sent to configured admin(s)
	 *
	 * @param string template - Template to test for
	 * @return mixed|array - Email to send test emails to
	 */
	public static function testmail($template) {
		$config = Kohana::$config->load('debug')->email_admin_only;

		if (is_null($config) OR ! is_array($config) OR empty($config[$template]))
			return FALSE;
		else
			return $config[$template];
	}

	public static function theme() {
		// If we are using user admin pages (and login), we'll choose the admin theme.
		return 'theme/'.(URL::admin_url() ? Kohana::$config->load('config')->theme_admin : Kohana::$config->load('config')->theme);
	}

	/**
	 * Show a date using a site configured format
	 */
	public static function time($date) {
		return date(Company::instance()->time_format(),($date ? $date : time()));
	}
}
?>
