<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 *
 * @package    OSB
 * @category   Models
 * @author     Deon George
 * @copyright  (c) 2009-2013 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 */
class Model_Group extends Model_Auth_Role {
	// Relationships
	protected $_has_many = array(
		'account'=>array('through'=>'account_group'),
		'module_method'=>array('through'=>'group_method','far_key'=>'method_id'),
	);
	protected $_sorting = array(
		'name'=>'ASC',
	);

	protected $_display_filters = array(
		'status'=>array(
			array('StaticList_YesNo::get',array(':value',TRUE)),
		),
	);

	/**
	 * This function will, given a group, list all of the children that
	 * are also related to this group, in the group heirarchy.
	 */
	public function list_childgrps($incParent=FALSE) {
		$result = array();

		if (! $this->loaded())
			return $result;

		foreach (ORM::factory('Group')->where_active()->and_where('parent_id','=',$this)->find_all() as $go) {
			array_push($result,$go);

			$result = array_merge($result,$go->list_childgrps());
		}

		if ($incParent)
			array_push($result,$this);

		return $result;
	}

	/**
	 * This function will, given a group, list all of the parent that
	 * are also related to this group, in the group heirarchy.
	 */
	public function list_parentgrps($incParent=FALSE) {
		$result = array();

		if (! $this->loaded())
			return $result;

		foreach (ORM::factory('Group')->where_active()->and_where('id','=',$this->parent_id)->find_all() as $go) {
			array_push($result,$go);

			$result = array_merge($result,$go->list_parentgrps());
		}

		if ($incParent)
			array_push($result,$this);

		return $result;
	}

	/**
	 * Get a list of groups that have their own pricing
	 */
	public function list_pricegroups() {
		return $this->where_active()->where('pricing','=',TRUE)->find_all();
	}
}
?>
