<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class supports Services
 *
 * @package    Host
 * @category   Models
 * @author     Deon George
 * @copyright  (c) 2009-2013 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 */
class Model_Service_Plugin_Host extends Model_Service_Plugin {
	protected $_table_name = 'service__hosting';
	protected $_created_column = FALSE;
	protected $_updated_column = FALSE;
	protected $_serialize_column = array(
		'server_data_date',
	);

	// Relationships
	protected $_has_one = array(
		'host'=>array('model'=>'Host_Server','far_key'=>'host_server_id','foreign_key'=>'id'),
		'tld'=>array('model'=>'Domain_TLD','foreign_key'=>'id','far_key'=>'domain_tld_id'),
	);
	protected $_belongs_to = array(
		'service'=>array(),
	);

	protected $_display_filters = array(
		'domain_name'=>array(
			array('strtoupper',array(':value')),
		),
		'host_expire'=>array(
			array('Config::date',array(':value')),
		),
	);

	protected $_save_message = TRUE;

	// Required abstract functions

	public function expire() {
		return $this->host_expire;
	}

	public function name() {
		return sprintf('%s.%s',$this->display('domain_name'),$this->tld->display('name'));
	}

	public function password() {
		return $this->host_password;
	}

	public function username() {
		return $this->host_username;
	}

	// Local functions

	/**
	 * This provides us with a manage button to jump to the hosting server
	 * to manage the domain.
	 */
	public function manage_button($t='') {
		// @todo Convert this to a Static_List display
		if ($this->service->queue == 'PROVISION')
			return _('To Be Provisioned');

		if (! parent::manage($t))
			return NULL;

		return ($this->username() AND $this->password() AND $a=$this->host->plugin()) ? $a->manage_button($this,$t) : '';
	}

	/**
	 * Search for services matching a term
	 */
	public function list_autocomplete($term,$index,$value,array $label,array $limit=array(),array $options=NULL) {
		// We only show domain names.
		if (is_numeric($term))
			return array();

		$ao = Auth::instance()->get_user();

		$options['key'] = 'id';
		$options['object'] = DB::select($this->_table_name.'.id',$this->_table_name.'.domain_name')
			->from($this->_table_name)
			->join('service')
			->on('service.id','=',$this->_table_name.'.service_id')
			->where('service.account_id','IN',$ao->RTM->customers($ao->RTM))
			->and_where($this->_table_name.'.domain_name','like','%'.$term.'%');

		return parent::list_autocomplete($term,$index,$value,$label,$limit,$options);
	}
}
?>
